using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;
using LinearProgramming;

namespace ALM
{
    /// <summary>
    /// The Tool used to change the Guis Layout.
    /// Uses the input from  the user to alter the shape and postion of the areas and 
    /// their content via the properties in the edit toolbar or from the mouse
    /// </summary>
    public partial class ALMEditor : Control
    {
        private System.Windows.Forms.ContextMenuStrip rightClickMenu;
        private System.Windows.Forms.ToolStripMenuItem removeContentMenuItem;
        private System.Windows.Forms.ToolStripMenuItem removeRowMenuItem;
        private System.Windows.Forms.ToolStripMenuItem splitAreaHorizontalMenuItem;
        private System.Windows.Forms.ToolStripMenuItem splitAreaVerticalMenuItem;
        private System.Windows.Forms.ToolStripMenuItem removeColumnMenuItem;
        private System.Windows.Forms.ToolStripMenuItem splitRowMenuItem;
        private System.Windows.Forms.ToolStripMenuItem splitColumnMenuItem;
        private System.Windows.Forms.ToolStripMenuItem addConstraintTabMenuItem;
        private System.Windows.Forms.ToolStripMenuItem removeConstraintTabItem;
        public enum Mode { AreaEdit, ConstraintEdit, RowEdit, ColumnEdit };
        ALMEngine aLMEngine;
        public List<Variable> selectedTabs;
        public Variable selectedXTab;
        public Variable selectedTab;
        public Variable selectedYTab;
        public Row selectedRow;
        public Row mouseOverRow;
        public Column selectedColumn;
        public Column mouseOverColumn;
        public Constraint selectedConstraint;
        public bool showXTab;
        public bool showYTab;
        Variable mouseOverXTab;
        Variable mouseOverYTab;
        public Area selectedArea;
        Area mouseOverArea;
        //boolean values to determine mouse cursor shape when moving an area by ts corner
        bool moveNESW, moveNWSE;
        public Mode mode;
        //Tolerance value for mouse clicks
        const int TOL = 4;

        public ALMEditor(Mode mode, ALMEngine aLMEngine)
        {
            this.aLMEngine = aLMEngine;
            this.mode = mode;
            if (selectedArea == null && selectedXTab == null && selectedYTab == null)
            {
                selectedArea = this.aLMEngine.LayoutSpec.Areas[0];
                selectedRow = selectedArea.Row;
                selectedColumn = selectedArea.Column;
            }
            selectedTabs = new List<Variable>();
            this.MouseMove += new MouseEventHandler(ALMEditor_MouseMove);
            this.MouseDown += new MouseEventHandler(ALMEditor_MouseDown);
            this.MouseUp += new MouseEventHandler(ALMEditor_MouseUp);
            this.SetStyle(
            ControlStyles.AllPaintingInWmPaint |
            ControlStyles.UserPaint |
            ControlStyles.DoubleBuffer, true);

            this.rightClickMenu = new System.Windows.Forms.ContextMenuStrip();
            rightClickMenu.Size = new System.Drawing.Size(292, 24);
            rightClickMenu.TabIndex = 0;
            rightClickMenu.Text = "Right Click Menu";
            //this.SuspendLayout();
            // 
            // rightClickMenu
            // 
            this.rightClickMenu.Name = "Edit Menu";

            // ALMEditor
            // 
            this.ContextMenuStrip = this.rightClickMenu;
            this.ContextMenuStrip.SuspendLayout();
            this.SuspendLayout();

            // Menu Item for removing area content
            // 

            this.removeContentMenuItem = new ToolStripMenuItem();
            this.removeContentMenuItem.Name = "Remove Area Content";
            this.removeContentMenuItem.Size = new System.Drawing.Size(168, 22);
            this.removeContentMenuItem.Text = "Remove Area Content";
            this.removeContentMenuItem.MouseDown += new MouseEventHandler(removeContentMenuItem_MouseDown);
            // 
            // Menu Item for splitting an area Horizontally
            // 
            this.splitAreaHorizontalMenuItem = new ToolStripMenuItem();
            this.splitAreaHorizontalMenuItem.Name = "Split Area Horizontally";
            this.splitAreaHorizontalMenuItem.Size = new System.Drawing.Size(168, 22);
            this.splitAreaHorizontalMenuItem.Text = "Split Area Horizontally";
            this.splitAreaHorizontalMenuItem.MouseDown += new MouseEventHandler(splitAreaHorizontalMenuItem_MouseDown);
            // Menu Item for splitting an area Vertically
            // 
            this.splitAreaVerticalMenuItem = new ToolStripMenuItem();
            this.splitAreaVerticalMenuItem.Name = "Split Area Vertically";
            this.splitAreaVerticalMenuItem.Size = new System.Drawing.Size(168, 22);
            this.splitAreaVerticalMenuItem.Text = "Split Area Vertically";
            this.splitAreaVerticalMenuItem.MouseDown += new MouseEventHandler(splitAreaVerticalMenuItem_MouseDown);
            //
            // Menu Item for splitting a row
            // 
            this.splitRowMenuItem = new ToolStripMenuItem();
            this.splitRowMenuItem.Name = "Split Row";
            this.splitRowMenuItem.Size = new System.Drawing.Size(168, 22);
            this.splitRowMenuItem.Text = "Split Row";
            this.splitRowMenuItem.MouseDown += new MouseEventHandler(splitRowMenuItem_MouseDown);

            // Menu Item for removing a row

            this.removeRowMenuItem = new ToolStripMenuItem();
            this.removeRowMenuItem.Name = "Remove Row";
            this.removeRowMenuItem.Size = new System.Drawing.Size(168, 22);
            this.removeRowMenuItem.Text = "Remove Row";
            this.removeRowMenuItem.MouseDown += new MouseEventHandler(removeRowMenuItem_MouseDown);
            // 
            // Menu Item for removing a column

            this.removeColumnMenuItem = new ToolStripMenuItem();
            this.removeColumnMenuItem.Name = "Remove Column";
            this.removeColumnMenuItem.Size = new System.Drawing.Size(168, 22);
            this.removeColumnMenuItem.Text = "Remove Column";
            this.removeColumnMenuItem.MouseDown += new MouseEventHandler(removeColumnMenuItem_MouseDown);
            // 
            // Menu Item for splitting a column 
            // 
            this.splitColumnMenuItem = new ToolStripMenuItem();
            this.splitColumnMenuItem.Name = "Split Column";
            this.splitColumnMenuItem.Size = new System.Drawing.Size(168, 22);
            this.splitColumnMenuItem.Text = "Split Column";
            this.splitColumnMenuItem.MouseDown += new MouseEventHandler(splitColumnMenuItem_MouseDown);
            //
            // Menu Item for adding a tab to a constraint
            //
            this.addConstraintTabMenuItem = new ToolStripMenuItem();
            this.addConstraintTabMenuItem.Name = "Add Tab to Constarint";
            this.addConstraintTabMenuItem.Size = new System.Drawing.Size(168, 22);
            this.addConstraintTabMenuItem.Text = "Add Tab to Cosntraint";
            this.addConstraintTabMenuItem.MouseDown += new MouseEventHandler(addConstraintTabMenuItem_MouseDown);
            //
            // Menu Item for removing a tab from a constraint
            //
            this.removeConstraintTabItem = new ToolStripMenuItem();
            this.removeConstraintTabItem.Name = "Remove Tab from Constarint";
            this.removeConstraintTabItem.Size = new System.Drawing.Size(168, 22);
            this.removeConstraintTabItem.Text = "Remove Tab from Cosntraint";
            this.removeConstraintTabItem.MouseDown += new MouseEventHandler(removeConstraintTabItem_MouseDown);
            // rightClickMenu
            // 

            this.rightClickMenu.Name = "Edit menu";
            this.rightClickMenu.Size = new System.Drawing.Size(61, 4);
            // 
            this.ContextMenuStrip.ResumeLayout(false);
            this.ResumeLayout(false);
        }
        /// <summary>
        /// Determines the selection of Area,Constraints,Rows and Columns.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void ALMEditor_MouseDown(object sender, MouseEventArgs e)
        {
            //If in Area mode the area over which the mouse cursor is clicked(if there is one)
            // will become selected  as well as any tabs if they are present.
            if (this.mode == Mode.AreaEdit)
            {
                this.selectedArea = findSelectedArea(new Point(e.X, e.Y));
                if (this.selectedArea != null)
                {
                    this.selectedXTab = getAreaXTabNearPoint(selectedArea, new Point(e.X, e.Y), TOL);
                    this.selectedYTab = getAreaYTabNearPoint(selectedArea, new Point(e.X, e.Y), TOL);
                }
                if (selectedXTab != null)
                {
                    this.showXTab = true;
                }
                if (selectedYTab != null)
                {
                    this.showYTab = true;
                }

                this.Invalidate();
                this.aLMEngine.editForm.UpdateEditForm();
            }
            // In Constraint Mode, the constraint the cursor is currently over 
            //  will be selected. 
            else if (this.mode == Mode.ConstraintEdit)
            {
                this.aLMEngine.LayoutSpec.Save("presave.txt");
                bool removeConstTab = false;
                selectedTab = GetTabNearPoint(new Point(e.X, e.Y), TOL);
            }
            // In Row Mode, the constraint the cursor is currently over 
            //  will be selected. 
            else if (this.mode == Mode.RowEdit)
            {
                selectedRow = findSelectedRow(new Point(e.X, e.Y));
                this.Invalidate();
                this.aLMEngine.editForm.UpdateEditForm();
                this.aLMEngine.parent.PerformLayout();
            }
            // In Column Mode, the constraint the cursor is currently over 
            //  will be selected. 
            else if (this.mode == Mode.ColumnEdit)
            {
                selectedColumn = findSelectedColumn(new Point(e.X, e.Y));
                this.Invalidate();
                this.aLMEngine.editForm.UpdateEditForm();
                this.aLMEngine.parent.PerformLayout();
            }

        }
        /// <summary>
        /// Depending on the editing mode, swapping of either areas,tabs,row or columns takes place 
        /// once the mouse button is released. In the case of constraints the new position of the tabs
        /// after dragging is evaluated and the layout method called. 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void ALMEditor_MouseUp(object sender, MouseEventArgs e)
        {
            //if(this.aLMEngine.editForm.
            moveNESW = false;
            moveNWSE = false;
            if (this.mode == Mode.AreaEdit)
            {
                if (selectedArea != null)
                {
                    // If there are no tabs selected,Swap the mouseover and the selected areas
                    if (selectedXTab == null && selectedYTab == null
                        && mouseOverArea != null
                        && mouseOverArea != selectedArea)
                    {
                        XTab left = this.selectedArea.Left;
                        XTab right = this.selectedArea.Right;
                        YTab top = this.selectedArea.Top;
                        YTab bottom = this.selectedArea.Bottom;
                        this.selectedArea.Top = this.mouseOverArea.Top;
                        this.selectedArea.Left = this.mouseOverArea.Left;
                        this.selectedArea.Right = this.mouseOverArea.Right;
                        this.selectedArea.Bottom = this.mouseOverArea.Bottom;
                        this.mouseOverArea.Top = top;
                        this.mouseOverArea.Left = left;
                        this.mouseOverArea.Right = right;
                        this.mouseOverArea.Bottom = bottom;
                    }
                    //If there is a xtab selected, replace it with the mouseoverXTab
                    if (selectedXTab != null && mouseOverXTab != null)
                    {
                        if (selectedXTab == selectedArea.Left)
                            selectedArea.Left = (XTab)mouseOverXTab;
                        else
                            selectedArea.Right = (XTab)mouseOverXTab;
                    }
                    //If there is a ytab selected, replace it with the mouseoverYTab
                    if (selectedYTab != null && mouseOverYTab != null)
                    {
                        if (selectedYTab == selectedArea.Top)
                            selectedArea.Top = (YTab)mouseOverYTab;
                        else
                            selectedArea.Bottom = (YTab)mouseOverYTab;
                    }
                }
                if (!this.aLMEngine.editForm.showYTabBox.Checked)
                    this.showYTab = false;
                if (!this.aLMEngine.editForm.showXTabBox.Checked)
                    this.showXTab = false;
                this.mouseOverArea = null;
                this.mouseOverXTab = null;
                this.mouseOverYTab = null;
                this.selectedXTab = null;
                this.selectedYTab = null;
                this.Parent.PerformLayout();
                this.Invalidate();
                this.aLMEngine.editForm.UpdateEditForm();
            }
            else if (this.mode == Mode.ConstraintEdit)
            {
                //Check if selected Tab is a part of the selected Constraint
                if (selectedTab != null && selectedTabs.Contains(selectedTab))
                {
                    // In the case of a single variable in the constraint, change 
                    // the value of the right hand side to be the value of the tab
                    //
                    if (this.selectedConstraint.LeftSide.Length == 1)
                        this.selectedConstraint.RightSide = this.selectedTab.Value;
                    // In the case of two variables in the constraint, change 
                    // the value of the right hand side to be the difference between 
                    // the values of the two tabs
                    //
                    if (this.selectedConstraint.LeftSide.Length == 2)
                        this.selectedConstraint.RightSide = this.selectedConstraint.LeftSide[1].Var.Value - this.selectedConstraint.LeftSide[0].Var.Value;
                    this.aLMEngine.editForm.UpdateEditForm();
                    this.Parent.PerformLayout();
                    this.Invalidate();
                    this.aLMEngine.LayoutSpec.Save("save.txt");
                }
            }
            else if (this.mode == Mode.RowEdit)
            {
                // If the selected row and the current row the mouse is over are not 
                // then insert the selected row before the row.From the users 
                // perspective the gap where the selected row will fit is highlighted.
                if (mouseOverRow != null && selectedRow != null)
                {
                    selectedRow.Unlink();
                    this.selectedRow.InsertBefore(mouseOverRow);
                    this.mouseOverRow = null;
                    this.Parent.PerformLayout();
                    this.Invalidate();
                }
            }

            else if (this.mode == Mode.ColumnEdit)
            {
                // If the selected column and the current column the mouse is over are not 
                // then insert the selected column before the column.From the users 
                // perspective the gap where the selected column will fit is highlighted.
                if (mouseOverColumn != null && selectedColumn != null)
                {

                    selectedColumn.Unlink();
                    this.selectedColumn.InsertBefore(mouseOverColumn);
                    this.mouseOverColumn = null;
                    this.Parent.PerformLayout();
                    this.Invalidate();
                }
            }
        }
        /// <summary>
        /// Allows for the drag and drop functionality for Areas, Rows,Constraints
        /// and columns.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void ALMEditor_MouseMove(object sender, MouseEventArgs e)
        {
                // Check if the mouse is being dragged over a valid area and highlight it.
                // If a tab is being dragged, display all tabs of the same type in the form. 
                if (this.mode == Mode.AreaEdit)
                {
                    if (e.Button == MouseButtons.Left && selectedArea != null)
                    {
                        this.mouseOverArea = findSelectedArea(new Point(e.X, e.Y));
                        if (mouseOverArea == selectedArea)
                            mouseOverArea = null;
                        this.mouseOverXTab = GetXTabNearPoint(new Point(e.X, e.Y), TOL);
                        this.mouseOverYTab = GetYTabNearPoint(new Point(e.X, e.Y), TOL);
                        if (!this.showXTab && !this.showYTab)
                            Cursor.Current = Cursors.SizeAll;
                        if (this.showXTab && !this.showYTab)
                            Cursor.Current = Cursors.SizeWE;
                        if (!this.showXTab && this.showYTab)
                            Cursor.Current = Cursors.SizeNS;
                        if (this.showXTab && this.showYTab)
                        {
                            if (this.moveNESW)
                                Cursor.Current = Cursors.SizeNESW;
                            else if (this.moveNWSE)
                                Cursor.Current = Cursors.SizeNWSE;
                        }
                    }
                    if (e.Button == MouseButtons.None && selectedArea != null)
                        IsNearAreaBorder(new Point(e.X, e.Y), selectedArea);
                }
                else if (this.mode == Mode.ConstraintEdit)
                {
                    if (this.selectedTab != null)
                    {
                        //Check if the selected Tab is part of the selected Constraint
                        if (this.selectedTabs.Contains(this.selectedTab))
                        {
                            // In the case of one variable in the constraint
                            if (selectedTab is XTab)
                                this.selectedTab.Value = e.X;
                            else
                                this.selectedTab.Value = e.Y;
                        }
                    }
                }
                // Highlight the gap that the selected row could fit into while being dragged.
                else if (this.mode == Mode.RowEdit)
                {
                    if (selectedArea != null)
                    {
                        foreach (Row row in this.aLMEngine.LayoutSpec.Rows)
                        {
                            if (this.GetYTabNearPoint(new Point(e.X, e.Y), TOL) != null && this.GetYTabNearPoint(new Point(e.X, e.Y), TOL).Value == row.top.Value)
                                mouseOverRow = row;
                        }
                    }
                    if (mouseOverRow == selectedRow || mouseOverRow == selectedRow.Next)
                        mouseOverRow = null;
                }
                // Highlight the gap that the selected column could fit into while being dragged. 
                else if (this.mode == Mode.ColumnEdit)
                {
                    if (selectedArea != null)
                    {
                        foreach (Column column in this.aLMEngine.LayoutSpec.Columns)
                        {
                            if (this.GetXTabNearPoint(new Point(e.X, e.Y), TOL) != null && this.GetXTabNearPoint(new Point(e.X, e.Y), TOL).Value == column.left.Value)
                                mouseOverColumn = column;
                        }
                    }
                    if (mouseOverColumn == selectedColumn || mouseOverColumn == selectedColumn.Next)
                        mouseOverColumn = null;
                }
            this.Invalidate();
        }

        /// <summary>
        /// Takes the current mouse coordinate and the selcted area and determines whether the
        /// cursor is near one of the borders of the selected area, if it is the cursor is changed
        /// to one of two possible indicator arrows
        /// </summary>
        /// <param name="p">Current Mouse Location </param>
        /// <param name="a">Currently Selected Area</param>
        void IsNearAreaBorder(Point p, Area a)
        {
            bool showHorizontalArrow = false;
            bool showVerticalArrow = false;

            if (p.Y >= a.Top.Value - TOL && p.Y <= a.Bottom.Value + TOL)
            {
                if (p.X <= a.Left.Value + TOL && p.X >= a.Left.Value)
                    showHorizontalArrow = true;
                else if (p.X <= a.Right.Value && p.X >= a.Right.Value - TOL)
                    showHorizontalArrow = true;
            }

            if (p.X >= a.Left.Value - TOL && p.X <= a.Right.Value + TOL)
            {
                if (p.Y <= a.Top.Value + TOL && p.Y >= a.Top.Value)
                    showVerticalArrow = true;
                else if (p.Y <= a.Bottom.Value && p.Y >= a.Bottom.Value - TOL)
                    showVerticalArrow = true;
            }
            if (showHorizontalArrow)
                Cursor.Current = Cursors.SizeWE;
            if (showVerticalArrow)
                Cursor.Current = Cursors.SizeNS;
            if (showHorizontalArrow && showVerticalArrow)
            {
                if (p.Y >= a.Top.Value - TOL && p.X >= a.Right.Value - TOL
                    && p.Y < a.Bottom.Value - TOL && p.X >= a.Left.Value + TOL ||
                    (p.Y >= a.Bottom.Value - TOL && p.X >= a.Left.Value - TOL
                    && p.Y > a.Top.Value + TOL && p.X <= a.Right.Value - TOL)
                    )
                {
                    Cursor.Current = Cursors.SizeNESW;
                    moveNESW = true;
                }
                else
                {
                    Cursor.Current = Cursors.SizeNWSE;
                    moveNWSE = true;
                }



            }
        }

        /// <summary>
        /// Determines which area contains the cursor
        /// </summary>
        /// <param name="point">Current Mouse location</param>
        /// <returns></returns>
        protected Area findSelectedArea(Point point)
        {
            foreach (Area a in this.aLMEngine.LayoutSpec.Areas)
            {
                if (new Rectangle((int)a.Left.Value, (int)a.Top.Value,
                    (int)(a.Right.Value - a.Left.Value), (int)(a.Bottom.Value - a.Top.Value)).Contains(point))
                {
                    return a;
                }
            }
            return null;
        }
        /// <summary>
        /// Determines which row contains the cursor
        /// </summary>
        /// <param name="point">Current Mouse Location</param>
        /// <returns></returns>
        protected Row findSelectedRow(Point point)
        {
            foreach (Row row in this.aLMEngine.LayoutSpec.Rows)
            {
                //if (this.GetYTabNearPoint(point, TOL) != null && this.GetYTabNearPoint(point, TOL).Value == row.top.Value)
                if (new Rectangle(0, (int)row.Top.Value, this.Width - 1, (int)row.Bottom.Value - (int)row.Top.Value).Contains(point))
                {
                    return row;
                }
            }
            return null;
        }
        protected Column findSelectedColumn(Point point)
        {
            foreach (Column column in this.aLMEngine.LayoutSpec.Columns)
            {
                //if (this.GetYTabNearPoint(point, TOL) != null && this.GetYTabNearPoint(point, TOL).Value == row.top.Value)
                if (new Rectangle((int)column.left.Value, 0, (int)column.right.Value - (int)column.left.Value, this.Height - 1).Contains(point))
                {
                    return column;
                }
            }
            return null;
        }
        /// <summary>
        /// Checks if there is a Xtab near where the mouse click takes place and returns it, if there is more than one tab 
        /// near the click area then the closest is returned. 
        /// </summary>
        /// <param name="p"></param>
        /// <param name="tolerance"></param>
        /// <returns></returns>
        Variable GetXTabNearPoint(Point p, int tolerance)
        {
            Variable selectedTab = null;
            int proximity = Int32.MaxValue;
            foreach (Variable v in aLMEngine.LayoutSpec.Variables)
            {
                if (v is XTab && v.Value + tolerance > p.X && p.X > v.Value - tolerance)
                {
                    if (Math.Abs(v.Value - p.X) < proximity)
                    {
                        selectedTab = v;
                        proximity = (int)Math.Abs(v.Value - p.X);
                        return selectedTab;
                    }
                }
            }
            return selectedTab;

        }

        /// <summary>
        /// Checks if there is a Ytab near where the mouse click takes place and returns it, if there is more than one tab 
        /// near the click area then the closest is returned. 
        /// </summary>
        /// <param name="p"></param>
        /// <param name="tolerance"></param>
        /// <returns></returns>
        /// 
        Variable GetYTabNearPoint(Point p, int tolerance)
        {
            Variable selectedTab = null;
            int proximity = Int32.MaxValue;
            foreach (Variable v in aLMEngine.LayoutSpec.Variables)
            {
                if (v is YTab && v.Value + tolerance > p.Y && p.Y > v.Value - tolerance)
                {
                    if (Math.Abs(v.Value - p.Y) < proximity)
                    {
                        selectedTab = v;
                        proximity = (int)Math.Abs(v.Value - p.Y);
                        return selectedTab;
                    }
                }
            }
            return selectedTab;
        }
        Variable getAreaXTabNearPoint(Area area, Point point, int tolerance)
        {
            XTab selectedTab = null;
            XTab nearestTab = (XTab)GetXTabNearPoint(point, tolerance);
            if (area.Right == nearestTab || area.Left == nearestTab)
                return nearestTab;
            else
                return null;
        }
        Variable getAreaYTabNearPoint(Area area, Point point, int tolerance)
        {
            YTab selectedTab = null;
            YTab nearestTab = (YTab)GetYTabNearPoint(point, tolerance);
            if (area.Top == nearestTab || area.Bottom == nearestTab)
                return nearestTab;
            else
                return null;
        }

        /// <summary>
        /// Checks if there is a tab near where the mouse click takes place and returns it, if there is more than one tab 
        /// near the click area then the closest is returned. 
        /// </summary>
        /// <param name="p"></param>
        /// <param name="tolerance"></param>
        /// <returns></returns>
        Variable GetTabNearPoint(Point p, int tolerance)
        {
            Variable selectedTab = null;
            int proximity = Int32.MaxValue;
            foreach (Variable v in aLMEngine.LayoutSpec.Variables)
            {
                if (v is XTab && v.Value + tolerance > p.X && p.X > v.Value - tolerance)
                {
                    if (Math.Abs(v.Value - p.X) < proximity)
                    {
                        selectedTab = v;
                        proximity = (int)Math.Abs(v.Value - p.X);
                    }
                }
                if (v is YTab && v.Value + tolerance > p.Y && p.Y > v.Value - tolerance)
                {
                    if (Math.Abs(v.Value - p.Y) < proximity)
                    {
                        selectedTab = v;
                        proximity = (int)Math.Abs(v.Value - p.Y);
                    }
                }
            }
            return selectedTab;
        }
        /// <summary>
        /// Copies all current coefficients and variables in the selected constraint into new arrays and then 
        /// adds the new tab with a coefficient of one to the new arrays, these then replace the old arrays.
        /// </summary>
        /// <param name="var"></param>
        private void addTabToConstraint(Variable var)
        {
            //this.selectedConstraint = (LinearProgramming.Constraint)this.aLMEngine.editForm.constraintBox.SelectedItem;
            //double[] coeffs = new double[selectedConstraint.LeftSide.Length + 1];
            //Variable[] vars = new Variable[selectedConstraint.LeftSide.Length + 1];
            Summand[] summands = new Summand[selectedConstraint.LeftSide.Length + 1];
            for (int i = 0; i < selectedConstraint.LeftSide.Length; i++)
            {
                //coeffs[i] = selectedConstraint.LeftSide[i].Coeff;
                //vars[i] = selectedConstraint.LeftSide[i].Var;
                summands[i].Coeff = selectedConstraint.LeftSide[i].Coeff;
                summands[i].Var = selectedConstraint.LeftSide[i].Var;
            }
            //coeffs[selectedConstraint.LeftSide.Length] = 1;
            //vars[selectedConstraint.LeftSide.Length] = var;
            summands[selectedConstraint.LeftSide.Length].Coeff = 1;
            summands[selectedConstraint.LeftSide.Length].Var = var;
            selectedConstraint.SetLeftSide(summands);
        }
        /// <summary>
        /// Copies all current coefficients and variables in the selected constraint into new arrays and then 
        /// removes the tab selected and its coefficient , these then replace the old arrays.
        /// </summary>
        /// <param name="var"></param>
        private void removeTabFromConstraint(Variable var)
        {
            //  LinearProgramming.Constraint selectedConstraint = (LinearProgramming.Constraint)this.aLMEngine.editForm.constraintBox.SelectedItem;
            //double[] coeffs = new double[selectedConstraint.Coeffs.Length - 1];
            //Variable[] vars = new Variable[selectedConstraint.Vars.Length - 1];
            Summand[] summands = new Summand[selectedConstraint.LeftSide.Length + 1];
            for (int i = 0, j = 0; i < selectedConstraint.LeftSide.Length; i++, j++)
            {
                if (selectedConstraint.LeftSide[i].Var == var)
                    j--;
                else
                {
                    //vars[j] = selectedConstraint.Vars[i];
                    //coeffs[j] = selectedConstraint.Coeffs[i];
                    summands[j] = selectedConstraint.LeftSide[i];
                }
            }
            selectedConstraint.SetLeftSide(summands);
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            Graphics graphics = e.Graphics;
            Pen normalPen = new Pen(Color.Blue, 2);
            Pen selectedPen = new Pen(Color.Red, 2);
            Pen destinationPen = new Pen(Color.Green, 4);

            //Paint all Areas
            foreach (Area a in this.aLMEngine.LayoutSpec.Areas)
            {
                Control control = a.Content;
                if (control == null)
                {
                    graphics.DrawRectangle(destinationPen,
                        (float)a.Left.Value, (float)a.Top.Value,
                        (float)(a.Right.Value - a.Left.Value),
                        (float)(a.Bottom.Value - a.Top.Value));
                    continue;
                }
                if (control.Width == 0 || control.Height == 0)
                    continue;
                Bitmap controlImage = new Bitmap(control.Width, control.Height);
                control.DrawToBitmap(controlImage, new Rectangle(0, 0, control.Width, control.Height));
                graphics.DrawImage(controlImage, control.Location);
            }

            if (this.mode == Mode.AreaEdit)
            {
                // paint tabs if the option is selected
                if (showXTab || showYTab)
                {
                    foreach (Variable var in this.aLMEngine.LayoutSpec.Variables)
                    {
                        if (showXTab && var is XTab)
                            graphics.DrawLine(normalPen, (float)var.Value, 0, (float)var.Value, this.Height - 1);
                        else if (showYTab && var is YTab)
                            graphics.DrawLine(normalPen, 0, (float)var.Value, this.Width - 1, (float)var.Value);
                    }
                }
                // paint selected area
                if (selectedArea != null)
                {
                    double left = selectedArea.Left.Value;
                    double top = selectedArea.Top.Value;
                    double right = selectedArea.Right.Value;
                    double bottom = selectedArea.Bottom.Value;
                    // draw selectedArea selection rectangle
                    graphics.DrawRectangle(selectedPen,
                        (float)selectedArea.Left.Value, (float)selectedArea.Top.Value,
                        (float)(selectedArea.Right.Value - selectedArea.Left.Value),
                        (float)(selectedArea.Bottom.Value - selectedArea.Top.Value));

                    // whole selectedArea is dragged
                    if (selectedXTab == null && selectedYTab == null
                        && mouseOverArea != null)
                    {
                        // draw destination rectangle
                        graphics.DrawRectangle(destinationPen,
                        (float)mouseOverArea.Left.Value, (float)mouseOverArea.Top.Value,
                        (float)(mouseOverArea.Right.Value - mouseOverArea.Left.Value),
                        (float)(mouseOverArea.Bottom.Value - mouseOverArea.Top.Value));
                    }
                    // tab(s) of selectedArea is dragged
                    else if (mouseOverXTab != null || mouseOverYTab != null)
                    {
                        // an xtab of the selectedArea is being dragged
                        if (selectedXTab != null && mouseOverXTab != null)
                        {
                            if (selectedXTab == selectedArea.Left)
                                left = mouseOverXTab.Value;
                            else
                                right = mouseOverXTab.Value;
                        }

                        // an ytab of the selectedArea is being dragged
                        if (selectedYTab != null && mouseOverYTab != null)
                        {
                            if (selectedYTab == selectedArea.Top)
                                top = mouseOverYTab.Value;
                            else
                                bottom = mouseOverYTab.Value;
                        }

                        // draw destination rectangle
                        graphics.DrawRectangle(destinationPen,
                        (float)left, (float)top,
                        (float)(right - left),
                        (float)(bottom - top));
                    }
                }
            }
            else if (this.mode == Mode.ConstraintEdit)
            {
                foreach (Variable var in this.aLMEngine.LayoutSpec.Variables)
                {
                    if (var is XTab && this.selectedTabs.Contains(var))
                    {
                        graphics.DrawLine(selectedPen, (float)var.Value, 0, (float)var.Value, this.Height - 1);
                    }
                    else if (var is YTab && this.selectedTabs.Contains(var))
                    {
                        graphics.DrawLine(selectedPen, 0, (float)var.Value, this.Width - 1, (float)var.Value);
                    }
                    else if (var is XTab)
                    {
                        graphics.DrawLine(normalPen, (float)var.Value, 0, (float)var.Value, this.Height - 1);
                    }
                    else if (var is YTab)
                    {
                        graphics.DrawLine(normalPen, 0, (float)var.Value, this.Width - 1, (float)var.Value);
                    }
                    if (var is XTab && this.selectedTab == var)
                    {
                        graphics.DrawLine(destinationPen, (float)var.Value, 0, (float)var.Value, this.Height - 1);
                    }
                    if (var is YTab && this.selectedTab == var)
                    {
                        graphics.DrawLine(destinationPen, 0, (float)var.Value, this.Width - 1, (float)var.Value);
                    }

                }
            }
            else if (this.mode == Mode.RowEdit)
            {
                foreach (Row row in this.aLMEngine.LayoutSpec.Rows)
                {
                    if (row == this.mouseOverRow)
                        graphics.DrawLine(destinationPen, 0, (float)mouseOverRow.top.Value, this.Width, (float)mouseOverRow.top.Value);
                    if (row == this.selectedRow)
                        graphics.DrawRectangle(selectedPen, 0, (float)row.Top.Value + 2, this.Width - 2, (float)row.Bottom.Value - (float)row.Top.Value - 5);

                    else
                        graphics.DrawRectangle(normalPen, 0, (float)row.Top.Value + 2, this.Width - 2, (float)row.Bottom.Value - (float)row.Top.Value - 5);
                }
            }
            else if (this.mode == Mode.ColumnEdit)
            {
                foreach (Column col in this.aLMEngine.LayoutSpec.Columns)
                {
                    if (col == this.mouseOverColumn)
                        graphics.DrawLine(destinationPen, (float)col.Left.Value, 0, (float)col.Left.Value, this.Height - 2);
                    if (col == this.selectedColumn)
                        graphics.DrawRectangle(selectedPen, (float)col.Left.Value + 2, 0, (float)col.Right.Value - (float)col.Left.Value - 5, this.Height - 2);
                    else
                        graphics.DrawRectangle(normalPen, (float)col.Left.Value + 2, 0, (float)col.Right.Value - (float)col.Left.Value - 5, this.Height - 2);
                }
            }
        }

        private void removeContentMenuItem_MouseDown(object sender, MouseEventArgs e)
        {
            //If the selected areas content is already null, remove it
            if (this.selectedArea.content == null)
                return;
            //Add the current areas content to the invisbleControls list
            this.aLMEngine.editForm.invisibleControls.Add(this.selectedArea.content);
            //Add the content as a selectable item in the control Box
            this.aLMEngine.editForm.controlBox.Items.Add(this.selectedArea.content.ToString().Substring(this.selectedArea.content.ToString().LastIndexOf(":") + 1));
            //Set the content as invisible
            this.selectedArea.content.Visible = false;
            //Remove the Content ie set it to null
            this.selectedArea.content = null;
            this.PerformLayout();
            this.aLMEngine.Layout();
            this.Invalidate();
            this.aLMEngine.editForm.UpdateEditForm();
        }

        void splitAreaHorizontalMenuItem_MouseDown(object sender, MouseEventArgs e)
        {
            YTab newTab = new YTab(this.aLMEngine.LayoutSpec);
            newTab.Value = this.selectedArea.Top.Value + (this.selectedArea.Bottom.Value - this.selectedArea.Top.Value) / 2;
            Area newArea = this.aLMEngine.LayoutSpec.AddArea(this.selectedArea.Left, this.selectedArea.Top, this.selectedArea.Right, newTab, null);
            newArea.PrefContentSize = new Size(this.selectedArea.PrefContentSize.Width / 2, this.selectedArea.PrefContentSize.Height / 2);
            newArea.AutoPrefContentSize = false;
            this.selectedArea.Top = newTab;
            this.aLMEngine.editForm.UpdateEditForm();
            this.PerformLayout();
            // this.aLMEngine.Layout();
            this.Invalidate();

        }

        // Allows for the splitting of the currently selected area along a vertical 
        // axis.
        void splitAreaVerticalMenuItem_MouseDown(object sender, MouseEventArgs e)
        {
            // The currently selected area is shrunk to half its width and forms the
            // left side of the split, a new area with the same dimensions of the 
            // area forms the right side of the split
            XTab newTab = new XTab(this.aLMEngine.LayoutSpec);
            newTab.Value = this.selectedArea.Left.Value + (this.selectedArea.Right.Value - this.selectedArea.Left.Value) / 2;
            Area newArea = this.aLMEngine.LayoutSpec.AddArea(newTab, this.selectedArea.Top, this.selectedArea.Right, this.selectedArea.Bottom, null);
            newArea.PrefContentSize = new Size(this.selectedArea.PrefContentSize.Width / 2, this.selectedArea.PrefContentSize.Height / 2);
            newArea.AutoPrefContentSize = false;
            this.selectedArea.Right = newTab;
            this.aLMEngine.editForm.UpdateEditForm();
            this.PerformLayout();
            //  this.aLMEngine.Layout();
            this.Invalidate();
        }

        //Allows for the splitting of the currently selected row
        void splitRowMenuItem_MouseDown(object sender, MouseEventArgs e)
        {
            YTab newTab = new YTab(this.aLMEngine.LayoutSpec);
            Row newRow = this.aLMEngine.LayoutSpec.AddRow();
            newRow.Unlink();
            newRow.bottom.Value = selectedRow.bottom.Value;
            newTab.Value = this.selectedRow.Top.Value + ((this.selectedRow.Bottom.Value - this.selectedRow.Top.Value) / 2);
            this.selectedRow.bottom.Value = newTab.Value;
            newRow.InsertAfter(selectedRow);
            newRow.bottom.Value = selectedRow.bottom.Value;

            this.aLMEngine.editForm.UpdateEditForm();
            this.PerformLayout();
            this.aLMEngine.Layout();
            this.Invalidate();

        }
        //Remove the selected row
        void removeRowMenuItem_MouseDown(object sender, MouseEventArgs e)
        {

            foreach (Area a in this.aLMEngine.LayoutSpec.Areas)
            {
                if (a.Row.Equals(this.selectedRow))
                {
                    Console.WriteLine("Area removed");
                    this.aLMEngine.editForm.invisibleControls.Add(this.selectedArea.content);
                    //Add the content as a selectable item in the control Box
                    this.aLMEngine.editForm.controlBox.Items.Add(this.selectedArea.content.ToString().Substring(this.selectedArea.content.ToString().LastIndexOf(":") + 1));
                    //Set the content as invisible
                    this.selectedArea.content.Visible = false;
                    //Remove the Content ie set it to null
                    this.selectedArea.content = null;
                }
            }
            this.selectedRow.Unlink();
            this.aLMEngine.editForm.UpdateEditForm();
            this.PerformLayout();
            this.aLMEngine.Layout();
            this.Invalidate();

        }
        //Remove the selected column
        void removeColumnMenuItem_MouseDown(object sender, MouseEventArgs e)
        {
            this.selectedColumn.Unlink();
            this.aLMEngine.editForm.UpdateEditForm();
            this.PerformLayout();
            this.aLMEngine.Layout();
            this.Invalidate();

        }

        void splitColumnMenuItem_MouseDown(object sender, MouseEventArgs e)
        {
        }

        // Adds the currently selected tab to the constraint.
        void removeConstraintTabItem_MouseDown(object sender, MouseEventArgs e)
        {
            if (this.selectedTab != null && this.selectedTabs.Contains(this.selectedTab) && this.selectedConstraint.LeftSide.Length != 1)
            {
                this.removeTabFromConstraint(selectedTab);
                this.aLMEngine.editForm.UpdateEditForm();
                this.Invalidate();
            }
        }
        // Adds the currently selected tab to the constraint.
        void addConstraintTabMenuItem_MouseDown(object sender, MouseEventArgs e)
        {
            if (this.selectedTab != null && !this.selectedTabs.Contains(this.selectedTab))
            {
                this.addTabToConstraint(selectedTab);
                this.aLMEngine.editForm.UpdateEditForm();
                this.Invalidate();
            }
        }
        /// <summary>
        /// Updates the right click menu with the suitable options for each mode
        /// </summary>
        public void updateRightClickMenu()
        {
            this.rightClickMenu.Items.Clear();
            if (this.mode == Mode.AreaEdit)
            {
                this.rightClickMenu.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.removeContentMenuItem});
                this.rightClickMenu.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.splitAreaHorizontalMenuItem});
                this.rightClickMenu.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.splitAreaVerticalMenuItem});
            }
            if (this.mode == Mode.ConstraintEdit)
            {
                this.rightClickMenu.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.addConstraintTabMenuItem});
                this.rightClickMenu.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.removeConstraintTabItem});
            }
            if (this.mode == Mode.RowEdit && aLMEngine.LayoutSpec.Rows.Count != 0)
            {
                this.rightClickMenu.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.removeRowMenuItem});
                this.rightClickMenu.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.splitRowMenuItem});

            }
            if (this.mode == Mode.ColumnEdit && aLMEngine.LayoutSpec.Columns.Count != 0)
            {
                this.rightClickMenu.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.splitColumnMenuItem});

                this.rightClickMenu.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.removeColumnMenuItem});
            }
        }
    }

}
