using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using LinearProgramming;

namespace ALM
{
    /// <summary>
    /// This form creates a toolbar that describes the attributes of the control 
    /// selected and allows for the adjustment of certain properties of selected 
    /// components. 
    /// </summary>
    public partial class EditForm : Form
    {
        const int TOL = 3;
        Control parent;
        ALMEngine aLMEngine;
        List<Control> savedEditControls;
        internal List<Control> invisibleControls;
        ALMEditor aLMEditor;

        public EditForm(Control parent, ALMEngine aLMEngine)
        {
            InitializeComponent();
            this.savedEditControls = new List<Control>();
            this.invisibleControls = new List<Control>();
            //Initialize attributes
            this.aLMEngine = aLMEngine;
            this.parent = parent;
            this.aLMEditor = new ALMEditor(ALMEditor.Mode.AreaEdit, this.aLMEngine);
        }
        /// <summary>
        /// Updates the edit form by detecting any changes made in the controls of 
        /// the editor(eg the currently selected area )and adjusting the attributes 
        /// of the various property boxes etc.
        /// </summary>
        public void UpdateEditForm()
        {
            //If the current mode selcted is Area Mode(the defaut mode on startup)
            //Initialize all content boxes/lists,clearing the old values
            if (this.aLMEditor.mode == ALMEditor.Mode.AreaEdit) // Areas tab
            {
                //Clear the old values
                areaBox.Items.Clear();
                leftBox.Items.Clear();
                rightBox.Items.Clear();
                topBox.Items.Clear();
                bottomBox.Items.Clear();
                controlBox.Items.Clear();
                areaRowBox.Items.Clear();
                areaColumnBox.Items.Clear();

                //Initialize the selection options           
                areaBox.Items.Add("New");
                // Loop through and add items to the controlox.
                foreach (Area area in aLMEngine.LayoutSpec.Areas)
                {
                    areaBox.Items.Add(area);
                }
                foreach (Control control in parent.Controls)
                {
                    //controlBox.Items.Add(control);.ToString().Substring(control.ToString().LastIndexOf(":") + 1));
                    this.savedEditControls.Add(control);
                }
                // Add the new Tab option for the Tab Boxes
                leftBox.Items.Add("New Tab");
                rightBox.Items.Add("New Tab");
                topBox.Items.Add("New Tab");
                bottomBox.Items.Add("New Tab");
                
                // Update the tab propery boxes with the current tabs
                foreach (Variable v in aLMEngine.LayoutSpec.Variables)
                {
                    if (v is XTab)
                    {
                        leftBox.Items.Add(v);
                        rightBox.Items.Add(v);
                    }
                    else if (v is YTab)
                    {
                        topBox.Items.Add(v);
                        bottomBox.Items.Add(v);
                    }
                }
                if (this.aLMEditor.selectedArea == null) return;
                controlBox.Items.Add("None");
                if (this.aLMEditor.selectedArea.content != null)
                {
                    // Add the names of the available controls to the list of available controls
                    this.controlBox.Items.Add(
                        this.aLMEditor.selectedArea.content.ToString().Substring(
                        this.aLMEditor.selectedArea.content.ToString().LastIndexOf(":") + 1));
                    controlBox.SelectedIndex = 1;
                }
                else
                    controlBox.SelectedIndex = 0;

                //Add only those controls which are not visible;
                foreach (Control control in this.invisibleControls)
                {
                    controlBox.Items.Add(control.ToString().Substring(control.ToString().LastIndexOf(":") + 1));
                }

                //Set the values of the boxes to the corresponding porperties of the currently selected Area
                //If selected Area has no content
                if (this.aLMEditor.selectedArea.content == null)
                {
                    controlBox.SelectedItem = "None";
                    prefWidthBox.Text = "0";
                    prefHeightBox.Text = "0";
                }
                else
                //Display the properties of the content of the selected area
                {
                    controlBox.Text = this.aLMEditor.selectedArea.content.ToString().Substring(this.aLMEditor.selectedArea.content.ToString().LastIndexOf(":") + 1);
                    prefWidthBox.Text = this.aLMEditor.selectedArea.content.PreferredSize.Width.ToString();
                    prefHeightBox.Text = this.aLMEditor.selectedArea.content.PreferredSize.Height.ToString();
                }
                //Show properties of the selected Area
                areaBox.SelectedItem = this.aLMEditor.selectedArea;
                leftBox.SelectedItem = this.aLMEditor.selectedArea.Left;
                topBox.SelectedItem = this.aLMEditor.selectedArea.Top;
                rightBox.SelectedItem = this.aLMEditor.selectedArea.Right;
                bottomBox.SelectedItem = this.aLMEditor.selectedArea.Bottom;
                hAlignBox.SelectedItem = this.aLMEditor.selectedArea.HAlignment;
                vAlignBox.SelectedItem = this.aLMEditor.selectedArea.VAlignment;
                leftMarBox.Text = this.aLMEditor.selectedArea.LeftMargin.ToString();
                topMarBox.Text = this.aLMEditor.selectedArea.TopMargin.ToString();
                rightMarBox.Text = this.aLMEditor.selectedArea.RightMargin.ToString();
                bottomMarBox.Text = this.aLMEditor.selectedArea.BottomMargin.ToString();
                if (this.aLMEditor.selectedArea.Row != null)
                    areaRowBox.SelectedItem = this.aLMEditor.selectedArea.Row;
                if (this.aLMEditor.selectedArea.Column != null)
                    areaColumnBox.SelectedItem = this.aLMEditor.selectedArea.Column;
                //Initialzie the areaRow box with the available rows
                foreach (Row row in aLMEngine.LayoutSpec.Rows)
                {                    
                    areaRowBox.Items.Add(row);
                }
                //Initialize the areaColumn box with the available Columns
                foreach (Column column in aLMEngine.LayoutSpec.Columns)
                {
                    areaColumnBox.Items.Add(column);
                }
                areaRowBox.SelectedItem = this.aLMEditor.selectedArea.Row;
                areaColumnBox.SelectedItem = this.aLMEditor.selectedArea.Column;
            }
            //If in constraint mode
            else if (this.aLMEditor.mode == ALMEditor.Mode.ConstraintEdit)
            {
                //Clear old selected Tabs
                this.aLMEditor.selectedTabs.Clear();
                // set slected Tabs to the ones in the currently selected Constrsint
                foreach (Summand s in this.aLMEditor.selectedConstraint.LeftSide)
                {
                    this.aLMEditor.selectedTabs.Add(s.Var);
                }
                //Display the equation of the constraint
                constEquationBox.Text = this.aLMEditor.selectedConstraint.ToString();
                //Determine whether the constraint is a soft constraint
                if (constraintBox.SelectedItem is LinearProgramming.Constraint)
                {
                    //If it is a soft Constraint, display its penalties in separate boxes
                    this.negPenaltyLabel.Visible = true;
                    this.negPenaltyTextBox.Visible = true;
                    this.posPenaltylabel.Visible = true;
                    this.posPenaltyTextBox.Visible = true;
                    constEquationBox.Text = constEquationBox.Text.Substring(0,
                   constEquationBox.Text.IndexOf(";") - 1);
                    posPenaltyTextBox.Text = ((LinearProgramming.Constraint)this.aLMEditor.selectedConstraint).GetPenaltyPos().ToString();
                    negPenaltyTextBox.Text = ((LinearProgramming.Constraint)this.aLMEditor.selectedConstraint).GetPenaltyNeg().ToString();
                }
                else
                {
                    // If the constraint is not soft then don't make the penalty property
                    // boxes visible
                    posPenaltyTextBox.Text = "0";
                    negPenaltyTextBox.Text = "0";
                    this.negPenaltyLabel.Visible = false;
                    this.negPenaltyTextBox.Visible = false;
                    this.posPenaltylabel.Visible = false;
                    this.posPenaltyTextBox.Visible = false;
                }
                // Reinitialize the values in the vailable constraint box
                constraintBox.Items.Clear();
                foreach (LinearProgramming.Constraint c in aLMEngine.LayoutSpec.Constraints)
                {
                    constraintBox.Items.Add(c);

                }
                constraintBox.SelectedItem = this.aLMEditor.selectedConstraint;
                this.aLMEditor.Invalidate();
            }
            else if (this.aLMEditor.mode == ALMEditor.Mode.RowEdit)
            {
                // Update the row list property box with available rows in current form
                //Clear the old list
                rowBox.Items.Clear();
                rowBox.Items.Add("New");
                foreach (Row row in aLMEngine.LayoutSpec.Rows)
                {
                    rowBox.Items.Add(row);                   
                }
                this.rowBox.SelectedItem = this.aLMEditor.selectedRow;
            }
            else if (this.aLMEditor.mode == ALMEditor.Mode.ColumnEdit)
            {
                // Update the column property box with the available columns in 
                // the current form
                // Clear the column list
                columnBox.Items.Clear();
                columnBox.Items.Add("New");                
                foreach (Column column in aLMEngine.LayoutSpec.Columns)
                {
                    columnBox.Items.Add(column);                    
                }
                this.columnBox.SelectedItem = this.aLMEditor.selectedColumn;
            }
        }
        /// <summary>
        /// Make the area selected in the dropbox the current;ly slected area, if the option selcected is "New" create
        /// a new area.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void areaBox_SelectionChangeCommitted(object sender, EventArgs e)
        {

            XTab right = null;
            YTab bottom = null;
            //Check if a new Area is to be created
            if (this.areaBox.SelectedItem.Equals("New"))
            {
                //Set the new areas initial tabs to copy that of the first area in the list
                right = this.aLMEngine.LayoutSpec.Areas[0].Right;
                bottom = this.aLMEngine.LayoutSpec.Areas[0].Bottom;
                Area newArea = this.aLMEngine.LayoutSpec.AddArea(this.aLMEngine.LayoutSpec.Left, this.aLMEngine.LayoutSpec.Top, right, bottom, new System.Windows.Forms.Button());
                areaBox.Items.Add(newArea);
                this.aLMEditor.selectedArea = newArea;

            }
            else
            {
               //Make the selected option the selected area
                this.aLMEditor.selectedArea = (Area)areaBox.SelectedItem;               
            }
            this.aLMEditor.PerformLayout();
            this.aLMEditor.Invalidate();
            this.UpdateEditForm();
        }
        
        // Allows for the changing of an Area's content
        private void controlBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            // check if old content is chosen again
            if ((this.controlBox.SelectedItem.Equals("None")
                    && this.aLMEditor.selectedArea.content == null)
                || (this.aLMEditor.selectedArea.content != null
                    && this.controlBox.SelectedIndex == 1))
                return;

            if (this.controlBox.SelectedItem.Equals("None"))
            {
                //Add the current areas content to the invisibleControls list
                this.invisibleControls.Add(this.aLMEditor.selectedArea.content);
                //Add the content as a selectable item in the control Box
                this.controlBox.Items.Add(this.aLMEditor.selectedArea.content.ToString().Substring(this.aLMEditor.selectedArea.content.ToString().LastIndexOf(":") + 1));
                //Set the content as invisible
                this.aLMEditor.selectedArea.content.Visible = false;
                //Remove the Content ie set it to null
                this.aLMEditor.selectedArea.content = null;
            }
            else
            {
                //If there is already content in the selected area, remove the existing content and replace it
                //with the one selected in the controlBox

                Control newContent;
                if (this.aLMEditor.selectedArea.content != null)
                {
                    newContent =
                        this.invisibleControls[this.controlBox.SelectedIndex - 2];

                    // remove old content
                    this.aLMEditor.selectedArea.content.Visible = false;
                    this.invisibleControls.Add(this.aLMEditor.selectedArea.content);
                }
                else
                    newContent =
                        this.invisibleControls[this.controlBox.SelectedIndex - 1];

                // insert new content
                newContent.Visible = true;
                this.invisibleControls.Remove(newContent);
                this.aLMEditor.selectedArea.content = newContent;
            }

            this.aLMEditor.PerformLayout();
            this.aLMEngine.Layout();
            this.aLMEditor.Invalidate();
            this.UpdateEditForm();
        }
        
        //Make the selected option the new left tab of the selected area or creates a new XTab
        private void leftBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            
            if (leftBox.SelectedItem.ToString().Equals("New Tab"))
            {
                this.aLMEditor.selectedArea.Left = aLMEngine.LayoutSpec.AddXTab();
                leftBox.Items.Add(this.aLMEditor.selectedArea.Left);
            }
            else
                this.aLMEditor.selectedArea.Left = (XTab)leftBox.SelectedItem;
            this.aLMEditor.Invalidate();
            parent.PerformLayout();
        }

        //Make the selected option the new top tab of the selected area or creates
        // a new Top tab
        private void topBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            if (!(topBox.SelectedItem.ToString().Equals("New Tab")))
            {
                this.aLMEditor.selectedArea.Top = aLMEngine.LayoutSpec.AddYTab();
                topBox.Items.Add(this.aLMEditor.selectedArea.Top);
            }
            else
                this.aLMEditor.selectedArea.Top = (YTab)topBox.SelectedItem;
            this.aLMEditor.Invalidate();
            parent.PerformLayout();
        }

        // Make the selected option the new right tab of the selected area or creates
        // a new right tab
        private void rightBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            if ((rightBox.SelectedItem.ToString().Equals("New Tab")))
            {
                this.aLMEditor.selectedArea.Right = aLMEngine.LayoutSpec.AddXTab();
                rightBox.Items.Add(this.aLMEditor.selectedArea.Right);
            }
            else
                this.aLMEditor.selectedArea.Right = (XTab)rightBox.SelectedItem;
            this.aLMEditor.Invalidate();
            parent.PerformLayout();

        }

        //Make the selected option the new bottom tab of the selected area or creates
        // a new bottom tab
        private void bottomBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            if ((bottomBox.SelectedItem.ToString().Equals("New Tab")))
            {
                this.aLMEditor.selectedArea.Bottom = aLMEngine.LayoutSpec.AddYTab();
                bottomBox.Items.Add(this.aLMEditor.selectedArea.Bottom);
            }
            else
                this.aLMEditor.selectedArea.Bottom = (YTab)bottomBox.SelectedItem;
            this.aLMEditor.Invalidate();
            parent.PerformLayout();


        }

        //Make the selected option the new horizontal alignment setting for the selected area
        private void hAlignBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            this.aLMEditor.selectedArea.HAlignment = (HorizontalAlignment)hAlignBox.SelectedItem;
            this.aLMEditor.Invalidate();
            parent.PerformLayout();
        }

        //Make the selected option the new vertical alignment setting for the selected area
        private void vAlignBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            this.aLMEditor.selectedArea.VAlignment = (VerticalAlignment)vAlignBox.SelectedItem;
            this.aLMEditor.Invalidate();
            parent.PerformLayout();
        }

        //Quit edir mode once the form is closed
        private void EditForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            aLMEngine.QuitEdit();
        }
        /// <summary>
        /// Once the tab for the Area Editor Control page is selected, contraint edit mode is activated,all previous controls are svaed, 
        /// and new constraint Controls are created.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void areaPage_Enter(object sender, EventArgs e)
        {
            parent.Controls.Clear();
            this.aLMEditor.mode = ALMEditor.Mode.AreaEdit;
            this.aLMEditor.Dock = DockStyle.Fill;
            this.aLMEditor.Size = parent.Size;
            hAlignBox.Items.AddRange(new object[] { HorizontalAlignment.LEFT, HorizontalAlignment.RIGHT, HorizontalAlignment.CENTER, HorizontalAlignment.FILL, HorizontalAlignment.NONE });
            vAlignBox.Items.AddRange(new object[] { VerticalAlignment.TOP, VerticalAlignment.BOTTOM, VerticalAlignment.CENTER, VerticalAlignment.FILL, VerticalAlignment.NONE });
            parent.Controls.Add(this.aLMEditor);
            this.aLMEditor.updateRightClickMenu();
            parent.PerformLayout();
        }
        /// <summary>
        /// Once the tab for the Constraint Control page is selected, contraint edit mode is activated,all previous controls are svaed, 
        /// and new constraint Controls are created.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void constraintPage_Enter(object sender, EventArgs e)
        {
            //Clear the current list
            parent.Controls.Clear();
            //Enter Constraint Mode
            this.aLMEditor.mode = ALMEditor.Mode.ConstraintEdit;
            this.aLMEditor.Dock = DockStyle.Fill;
            this.aLMEditor.Size = parent.Size;
            //Clear old values if any
            constraintBox.Items.Clear();
            this.aLMEditor.selectedTabs.Clear();
            //Initialize values of contraint box
            foreach (LinearProgramming.Constraint c in aLMEngine.LayoutSpec.Constraints)
            {
                constraintBox.Items.Add(c);

            }
            this.aLMEditor.selectedConstraint = (LinearProgramming.Constraint)constraintBox.Items[0];

            parent.Controls.Add(this.aLMEditor);
            this.UpdateEditForm();
            this.aLMEditor.updateRightClickMenu();
            parent.PerformLayout();
        }

        /// <summary>
        /// Once the tab for the Row Control page is selected, Row edit mode is activated.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rowPage_Enter(object sender, EventArgs e)
        {
            //Clear the current list
            parent.Controls.Clear();
            this.aLMEditor.mode = ALMEditor.Mode.RowEdit;
            this.aLMEditor.Dock = DockStyle.Fill;
            this.aLMEditor.Size = parent.Size;
            parent.Controls.Add(this.aLMEditor);
            parent.PerformLayout();
            this.aLMEditor.updateRightClickMenu();
            this.UpdateEditForm();
        }

        /// <summary>
        /// Once the tab for theColumn Control page is selected,Column edit mode is activated.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void columnPage_Enter(object sender, EventArgs e)
        {
            //Clear the current list
            parent.Controls.Clear();
            this.aLMEditor.mode = ALMEditor.Mode.ColumnEdit;
            this.aLMEditor.Dock = DockStyle.Fill;
            this.aLMEditor.Size = parent.Size;
            parent.Controls.Add(this.aLMEditor);
            parent.PerformLayout();
            this.aLMEditor.updateRightClickMenu();
            this.UpdateEditForm();
        }

        /// <summary>
        /// Set the current selcted tabs to be those belonging to the selected constraint option
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void constraintBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            this.aLMEditor.selectedConstraint = (LinearProgramming.Constraint)constraintBox.SelectedItem;            
            this.UpdateEditForm();
            this.aLMEditor.Invalidate();

        }

        /// <summary>
        /// Changes the preferred width of the selected area
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void prefWidthBox_TextChanged(object sender, EventArgs e)
        {
            if (prefWidthBox.Text.Equals("") || this.aLMEditor.selectedArea.
                PrefContentSize.Width == Convert.ToInt32(prefWidthBox.Text) ||
                this.aLMEditor.selectedArea.AutoPrefContentSize || prefHeightBox.Text.Equals("")) return;
            this.aLMEditor.selectedArea.PrefContentSize = new Size(Convert.ToInt32(prefWidthBox.Text),
                Convert.ToInt32(prefHeightBox.Text));
        }

        /// <summary>
        /// Changes the preferred width of the selected area
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void prefHeightBox_TextChanged(object sender, EventArgs e)
        {
            if (prefHeightBox.Text.Equals("") || this.aLMEditor.selectedArea.
                PrefContentSize.Height == Convert.ToInt32(prefHeightBox.Text) ||
                this.aLMEditor.selectedArea.AutoPrefContentSize) return;
            this.aLMEditor.selectedArea.PrefContentSize = new Size(Convert.ToInt32(prefWidthBox.Text),
            Convert.ToInt32(prefHeightBox.Text));
        }

        /// <summary>
        /// Changes the left margin of the selected area
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void leftMarBox_TextChanged(object sender, EventArgs e)
        {
            if (leftMarBox.Text.Equals("") || this.aLMEditor.selectedArea.LeftMargin == Convert.ToInt32(leftMarBox.Text)) return;
            this.aLMEditor.selectedArea.LeftMargin = Convert.ToInt32(leftMarBox.Text);
            this.aLMEditor.Invalidate();
            parent.PerformLayout();
        }

        /// <summary>
        /// Changes the top margin of the selected area
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void topMarBox_TextChanged(object sender, EventArgs e)
        {
            if (topMarBox.Text.Equals("") || this.aLMEditor.selectedArea.TopMargin == Convert.ToInt32(topMarBox.Text)) return;
            this.aLMEditor.selectedArea.TopMargin = Convert.ToInt32(topMarBox.Text);
            this.aLMEditor.Invalidate();
            parent.PerformLayout();
        }

        /// <summary>
        /// Changes the right margin of the selected area
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rightMarBox_TextChanged(object sender, EventArgs e)
        {
            if (rightMarBox.Text.Equals("") || this.aLMEditor.selectedArea.RightMargin == Convert.ToInt32(rightMarBox.Text)) return;
            this.aLMEditor.selectedArea.RightMargin = Convert.ToInt32(rightMarBox.Text);
            this.aLMEditor.Invalidate();
            parent.PerformLayout();
        }

        /// <summary>
        /// Changes the bottom margin of the selected area
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bottomMarBox_TextChanged(object sender, EventArgs e)
        {
            if (bottomMarBox.Text.Equals("") || this.aLMEditor.selectedArea.BottomMargin == Convert.ToInt32(bottomMarBox.Text)) return;
            this.aLMEditor.selectedArea.BottomMargin = Convert.ToInt32(bottomMarBox.Text);
            this.aLMEditor.Invalidate();
            parent.PerformLayout();
        }

        /// <summary>
        /// Show or clear away the current xTabs in the display depending on 
        /// whether the box is checked or not.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void showXTabBox_CheckedChanged(object sender, EventArgs e)
        {
            if (showXTabBox.Checked)
                this.aLMEditor.showXTab = true;
            else
                this.aLMEditor.showXTab = false;
            parent.PerformLayout();
            this.aLMEditor.Invalidate();

        }

        /// <summary>
        /// Show or clear away the current yTabs in the display depending on 
        /// whether the box is checked or not.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void showYTabBox_CheckedChanged(object sender, EventArgs e)
        {
            if (showYTabBox.Checked)
                this.aLMEditor.showYTab = true;
            else
                this.aLMEditor.showYTab = false;
            parent.PerformLayout();
            this.aLMEditor.Invalidate();
        }

        /// <summary>
        /// Make the selected Row the row chosen in the option box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rowBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            if (rowBox.SelectedItem.Equals("New"))
            {
                Row newRow = this.aLMEngine.LayoutSpec.AddRow();
                newRow.InsertAfter(this.aLMEngine.LayoutSpec.Rows[0]);
            }
            else
            {
                this.aLMEditor.selectedRow = (Row)this.rowBox.SelectedItem;
            }
            this.aLMEditor.Invalidate();
            parent.PerformLayout();
            this.UpdateEditForm();
        }

        /// <summary>
        /// Make the selected Column the column chosen in the option box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void columnBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            if (columnBox.SelectedItem.Equals("New"))
            {
                Column newColumn = this.aLMEngine.LayoutSpec.AddColumn();
                newColumn.InsertAfter(this.aLMEngine.LayoutSpec.Columns[0]);
            }
            else
            {
                this.aLMEditor.selectedColumn = (Column)this.columnBox.SelectedItem;

            }
            this.aLMEditor.Invalidate();
            parent.PerformLayout();
            this.UpdateEditForm();
        }

        /// <summary>
        /// Shift the selected area to the currently selected Row 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void areaRowBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            this.aLMEditor.selectedArea.Row = (Row)areaRowBox.SelectedItem;
            this.aLMEditor.Invalidate();
            parent.PerformLayout();
            this.UpdateEditForm();
        }

        /// <summary>
        /// Shift the selected area to the currently selected Column
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void areaColumnBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            this.aLMEditor.selectedArea.Column = (Column)areaColumnBox.SelectedItem;
            this.aLMEditor.Invalidate();
            parent.PerformLayout();
            this.UpdateEditForm();
        }
    }
}

