using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Windows.Forms.Layout;
using LinearProgramming;
using ALM;

namespace LayoutTest1
{
    public partial class Form1 : Form
    {
        ALMEngine le = new ALMEngine();

        public override LayoutEngine LayoutEngine
        {
            get { return le; }
        }

        public Form1()
        {
            InitializeComponent();

            le.LayoutSpec = Generate(100, 0);
            //testResize(301, 500, 10, 10);
            //testConstraints(400, 100, 20, 50);
        }

        LayoutSpec Generate(int n_areas, int n_constraints)
        {
            Random rnd = new Random();
            LayoutSpec l = new LayoutSpec();
            l.AddArea(l.Left, l.Top, l.Right, l.Bottom, null, new Size(0, 0));
            while (l.Areas.Count < n_areas)
            {
                Area a = l.Areas[rnd.Next(l.Areas.Count)];
                a.Remove();
                if (rnd.NextDouble() < 0.5)
                {
                    XTab x = l.AddXTab();
                    l.AddArea(a.Left, a.Top, x, a.Bottom, null, new Size(0, 0));
                    l.AddArea(x, a.Top, a.Right, a.Bottom, null, new Size(0, 0));
                }
                else
                {
                    YTab y = l.AddYTab();
                    l.AddArea(a.Left, a.Top, a.Right, y, null, new Size(0, 0));
                    l.AddArea(a.Left, y, a.Right, a.Bottom, null, new Size(0, 0));
                }
            }

            foreach (Area a in l.Areas)
            {
                Button b = new Button();
                b.Text = (l.Areas.IndexOf(a) + 1).ToString();
                this.Controls.Add(b);
                a.Content = b;
                a.PrefContentSize = new Size(16 + rnd.Next(80), 16 + rnd.Next(80));
                //a.ExpandRigidity = new SizeF((float)rnd.NextDouble(), (float)rnd.NextDouble());
                //a.ShrinkRigidity = new SizeF((float)rnd.NextDouble(), (float)rnd.NextDouble());

                //a.MinContentSize = new Size(rnd.Next(8 + 800 / n_areas), rnd.Next(8 + 600 / n_areas));
                //if (rnd.NextDouble() < 0.05)
                //{
                //    a.MaximumSize = new Size(
                //        a.MinimumSize.Width + rnd.Next(800),
                //        a.MinimumSize.Height + rnd.Next(600));
                //}
            }

            for (int i = 0; i < n_constraints; i++)
            {
                double[] coeffs = new double[1 + rnd.Next(4)];
                List<Variable> vars = new List<Variable>();
                for (int k = 0; k < coeffs.Length; k++)
                {
                    coeffs[k] = 0.5 - rnd.NextDouble();
                    Variable v;
                    do
                    {
                        v = l.Variables[rnd.Next(l.Variables.Count)];
                    } while (vars.Contains(v));
                    vars.Add(v);
                }
                l.AddSoftConstraint(coeffs, vars.ToArray(),
                        ((rnd.Next(2) == 0) ? OperatorType.EQ :
                            (rnd.Next(2) == 0) ? OperatorType.GE : OperatorType.LE),
                        0.5 - rnd.NextDouble(), rnd.NextDouble(), rnd.NextDouble());
            }
            return l;
        }

        void testResize(int areas_min, int areas_max, int n_layouts, int n_resizes)
        {
            StreamWriter sw = new StreamWriter("performance-resize.csv");
            sw.WriteLine("areas,vars,constraints,time,result,value");
            for (int a = areas_min; a < areas_max; a++)
            {
                for (int l = 0; l < n_layouts; l++)
                {
                    this.SuspendLayout();
                    le.LayoutSpec = Generate(a, 0);
                    this.ResumeLayout(false);

                    // first resize
                    Size = new Size(800, 600);
                    
                    Random rnd = new Random();
                    int[] d = new int[n_resizes];
                    for (int r = 0; r < n_resizes; r++) d[r] = 100 - rnd.Next(200);

                    DateTime t1 = DateTime.Now;
                    for (int r = 0; r < n_resizes; r++)
                    {
                        Size = new Size(800 + d[r], 600 + d[r]);
                    }
                    DateTime t2 = DateTime.Now;
                    TimeSpan t = t2 - t1;
                    double time = 100.0 * t.Ticks / (n_resizes * 1000000.0);
                    sw.WriteLine(le.LayoutSpec.Areas.Count + "," + le.LayoutSpec.Variables.Count + ","
                            + le.LayoutSpec.Constraints.Count + "," + time
                            + "," + le.LayoutSpec.Result + "," + le.LayoutSpec.ObjectiveValue);
                    sw.Flush();
                    le.LayoutSpec.Remove();
                }
            }
            sw.Close();
        }

        void testConstraints(int n_constraints, int n_areas, int n_layouts, int n_resizes)
        {
            StreamWriter sw = new StreamWriter("performance-constraints.csv");
            sw.WriteLine("areas,vars,constraints,time,result,value");
            for (int c = 100; c < n_constraints; c++)
            {
                for (int l = 0; l < n_layouts; l++)
                {
                    this.SuspendLayout();
                    le.LayoutSpec = Generate(n_areas, c);
                    this.ResumeLayout(false);

                    // first resize
                    Size = new Size(800, 600);

                    Random rnd = new Random();
                    int[] d = new int[n_resizes];
                    for (int r = 0; r < n_resizes; r++) d[r] = 100 - rnd.Next(200);

                    DateTime t1 = DateTime.Now;
                    for (int r = 0; r < n_resizes; r++)
                    {
                        Size = new Size(800 + d[r], 600 + d[r]);
                    }
                    DateTime t2 = DateTime.Now;
                    TimeSpan t = t2 - t1;
                    double time = 100.0 * t.Ticks / (n_resizes * 1000000.0);
                    sw.WriteLine(le.LayoutSpec.Areas.Count + "," + le.LayoutSpec.Variables.Count + ","
                            + le.LayoutSpec.Constraints.Count + "," + time
                            + "," + le.LayoutSpec.Result + "," + le.LayoutSpec.ObjectiveValue);
                    sw.Flush();
                    le.LayoutSpec.Remove();
                }
            }
            sw.Close();
        }
    }
}